/**
 * 基础拓展
 * @author yutent<yutent.io@gmail.com>
 * @date 2020/10/14 17:52:44
 */

const PLACEHOLDER = '在此输入文本'

function trim(str, sign) {
  return str.replace(new RegExp('^' + sign + '|' + sign + '$', 'g'), '')
}

export default {
  header() {
    this.$refs.header.classList.add('fadein')
  },

  h(level) {
    var wrap = this.selection(true) || PLACEHOLDER
    wrap = wrap.replace(/^(#+ )?/, '#'.repeat(level) + ' ')
    this.insert(wrap, true)
  },

  quote() {
    var wrap = this.selection(true) || PLACEHOLDER
    wrap = wrap.replace(/^(>+ )?/, '> ')

    this.insert(wrap, true)
  },

  bold() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '\\*\\*')
    wrap = wrap === unwrap ? `**${wrap}**` : unwrap
    this.insert(wrap, true)
  },

  italic() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '_')
    wrap = wrap === unwrap ? `_${wrap}_` : unwrap
    this.insert(wrap, true)
  },

  through() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '~~')
    wrap = wrap === unwrap ? `~~${wrap}~~` : unwrap
    this.insert(wrap, true)
  },

  list() {
    var wrap = this.selection(true) || PLACEHOLDER

    wrap = wrap.replace(/^([+\-*] )?/, '+ ')
    this.insert(wrap, true)
  },

  order() {
    var wrap = this.selection(true) || PLACEHOLDER

    wrap = wrap.replace(/^(\d+\. )?/, '1. ')
    this.insert(wrap, true)
  },

  line() {
    this.insert('\n\n---\n\n', false)
  },

  code() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '`')
    wrap = wrap === unwrap ? `\`${wrap}\`` : unwrap
    this.insert(wrap, true)
  },

  codeblock() {
    this.insert('\n```language\n\n```\n')
  },

  table() {
    this.$refs.table.classList.add('fadein')
  },

  link() {
    this.$refs.link.classList.add('fadein')
  },

  fullscreen() {
    this.classList.toggle('fullscreen')
  },
  preview() {
    this.previewEnabled = !this.previewEnabled
    this.$refs.view.classList.toggle('active')
    if (this.previewEnabled) {
      this.$refs.view.code = this.value
    }
  }
}
